@@ -5,69 +5,67 @@ import numpy as np
 from scipy.sparse.linalg import eigsh
 
 from .decorators import linear, real, symmetric, inplace
-from .core import (Operator, CompositionOperator, DiagonalOperator,
-                   DirectOperatorFactory, ReverseOperatorFactory, asoperator)
-from .utils import isscalar, strshape
+from .core import Operator, BlockRowOperator, CompositionOperator, DiagonalOperator, ReductionOperator, asoperator
+from .utils import isscalar
 
 __all__ = [
     'BandOperator',
-    'DenseOperator',
     'EigendecompositionOperator',
+    'IntegrationTrapezeWeightOperator',
     'PackOperator',
+    'SumOperator',
     'SymmetricBandOperator',
     'TridiagonalOperator',
     'UnpackOperator',
 ]
 
 
-@linear
-class DenseOperator(Operator):
+class IntegrationTrapezeWeightOperator(BlockRowOperator):
     """
-    Operator representing a dense matrix.
+    Return weights as a block row operator to perform trapeze integration.
+
+    This operator can be used to integrate over X the bivariate function 
+        f = f(X,Y).
+    Let's assume f is sampled at n abscissa x_n non necessarily equally spaced
+        f_i(Y) = f(x_i, Y).
+    The operator IntegrationTrapezeWeightOperator returns a block row operator
+        W = [ w_1 * I ... w_n * I]
+    such that, given the block column operator
+            [ f_1 ]
+        F = [ ... ]
+            [ f_n ],
+    the product
+        W * F = w_1 * f_1 + ... + w_n * f_n
+    performs a trapeze integration of f(X,Y) over the bins [x_i,x_(i+1)]
+    for i in 1..n-1.
 
     Example
     -------
-    >>> m = array([[1.,2.,3.],[2.,3.,4.]])
-    >>> d([1,0,0])
-    array([ 1.,  2.])
+    >>> f = np.power
+    >>> x = [0.5,1,2,4]
+    >>> F = BlockColumnOperator([Operator(lambda i,o,v=v:f(v,i,o),
+    ...                         flags='square') for v in x], new_axisout=0)
+    >>> W = IntegrationTrapezeWeightOperator(x)
+    >>> int_f = W * F
+    >>> int_f([0,1,2])
+    array([  3.5   ,   7.875 ,  22.8125])
+    >>> [ trapz(f(x,a), x) for a in [0,1,2] ]
+    [3.5, 7.875, 22.8125]
+
     """
-    def __init__(self, data, shapein=None, shapeout=None, dtype=None,
-                 **keywords):
-        if data is None:
-            raise ValueError('The input data is None.')
-        data = np.asarray(data)
-        if dtype is None:
-            dtype = data.dtype
-        data = np.array(data, dtype, copy=False)
-        if data.ndim != 2:
-            raise ValueError('The input is not a 2-dimensional array.')
-        if shapein is None:
-            shapein = data.shape[1]
-        elif np.product(shapein) != data.shape[1]:
-            raise ValueError("The input shape '{0}' is incompatible with that o"
-                             "f the input matrix '{1}'.".format(strshape(
-                             shapein), data.shape[1]))
-        if shapeout is None:
-            shapeout = data.shape[0]
-        elif np.product(shapeout) != data.shape[0]:
-            raise ValueError("The input shape '{0}' is incompatible with that o"
-                             "f the input matrix '{1}'.".format(strshape(
-                             shapeout), data.shape[0]))
-        Operator.__init__(self, shapein=shapein, shapeout=shapeout, dtype=dtype,
-                          **keywords)
-        self.data = data
-        self.set_rule('.C', lambda s: DirectOperatorFactory(type(s), s,
-            np.conjugate(s.data)))
-        self.set_rule('.T', lambda s: ReverseOperatorFactory(type(s), s,
-            s.data.T))
-        self.set_rule('.H', lambda s: ReverseOperatorFactory(type(s), s,
-            np.conjugate(s.data.T)))
+    def __init__(self, x, new_axisin=0, **keywords):
+        x = np.asarray(x)
+        if x.size < 2:
+            raise ValueError('At least two abscissa are required.')
+        if np.any(np.diff(x) < 0) and np.any(np.diff(x) > 0):
+            raise ValueError('The abscissa are not monotonous.')
 
-    def direct(self, input, output):
-        np.dot(self.data, input.ravel(), output)
-
-    def todense(self, shapein=None):
-        return self.data
+        w = np.empty_like(x)
+        w[0] = 0.5 * (x[1] - x[0])
+        w[1:-1] = 0.5 * (x[2:]-x[:-2])
+        w[-1] = 0.5 * (x[-1] - x[-2])
+        BlockRowOperator.__init__(self, list(w), new_axisin=new_axisin,
+                                  **keywords)
 
 
 @linear
@@ -108,6 +106,50 @@ class UnpackOperator(Operator):
         output[...] = 0
         output[self.mask] = input
 
+
+@linear
+class SumOperator(ReductionOperator):
+    """
+    Sum-along-axis operator.
+
+    Parameters
+    ----------
+    axis : integer, optional
+        Axis along which the reduction is performed. If None, all dimensions
+        are collapsed.
+    dtype : dtype, optional
+        Reduction data type.
+    skipna : boolean, optional
+        If this is set to True, the reduction is done as if any NA elements
+        were not counted in the array. The default, False, causes the NA values
+        to propagate, so if any element in a set of elements being reduced is
+        NA, the result will be NA.
+
+    Example
+    -------
+    >>> op = SumOperator()
+    >>> op([1,2,3])
+    array(6)
+
+    """
+    def __init__(self, axis=None, dtype=None, skipna=True, **keywords):
+        if np.__version__ < '1.7':
+            func = np.nansum if skipna else np.add
+        else:
+            func = np.add
+        ReductionOperator.__init__(self, func, axis=axis, dtype=dtype,
+                                   skipna=skipna, **keywords)
+
+    def transpose(self, input, output):
+        if self.axis is None:
+            shape = None
+        elif self.axis == -1:
+            shape = input.shape + (1,)
+        else:
+            shape = input.shape[:self.axis] + (1,) + input.shape[self.axis:]
+        output[...] = input.reshape(shape)
+
+
 @linear
 class TridiagonalOperator(Operator):
     def __init__(self, diag, subdiag, superdiag=None, **kwargs):
