@@ -11,6 +11,7 @@ import types
 from . import cythonutils as cu
 
 __all__ = ['all_eq',
+           'find',
            'first_is_not',
            'inspect_special_values',
            'isclassattr',
@@ -33,6 +34,8 @@ def all_eq(a, b):
     """
     Return True if a and b are equal by recursively comparing them.
     """
+    if a is b:
+        return True
     if isinstance(a, collections.Mapping):
         if type(a) is not type(b):
             return False
@@ -67,6 +70,26 @@ def all_eq(a, b):
         return a.func_code is b.func_code
     return a == b
 
+def find(l, f):
+    """
+    Return first item in list that verifies a certain condition, and None
+    otherwise.
+
+    Parameters
+    ----------
+    l : list
+        List of elements to be searched for.
+    f : function
+        Function that evaluates to True to match an element.
+
+    Example:
+    --------
+    >>> find([1,2,3], lambda x: x > 1.5)
+    2
+
+    """
+    return next((_ for _ in l if f(_)), None)
+
 def first_is_not(l, v):
     """
     Return first item in list which is not the specified value.
